%	SENSE REconstruction Demo
%	To run it, type sensedemo at the >> prompt.
%
%	written by Hsu-Lei Lee  Nov 2005

echo on
clc

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%	DEMO: 2DFT SENSE reconstruction  
%
%   This is a simple demonstration of SENSE reconstruction in 2DFT case.
%   
%	In this demo, we use phantom images and simulated coil sensitivity 
%   profiles generated with MATLAB.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%
%	Load coil sensitivity information and create a phantom object:
%
%	We simulate the sensitivity map of a 4-coil array; and each coil 
%   has different y position. The images obtained by these coils will
%   be the original object times the coil sensitivity map. In figure 1, 
%   the image in the 1st row is the original object, those in the 2nd 
%   row are the coil sensitivity maps. In the 3rd row we see the images 
%   obtained with each single coil (with full k-space data).
%

pause	% Press any key to begin

load reducfigure;
load coilprofile;
FOV = size(coilprofile,1);
coiln = size(coilprofile,3);
object = phantom('Modified Shepp-Logan',FOV);
coilimagefull = coilprofile;
figure(1);
subplot(3,4,1);ylabel('y');dispimage(object);title('Object');
for m = 1:coiln
    coilimage(:,:,m) = object.*coilprofile(:,:,m);
    subplot(3,4,4+m);dispimage(coilprofile(:,:,m));
    title1 = ['Coil #' num2str(m)];title(title1);
    subplot(3,4,8+m);dispimage(coilimage(:,:,m));
    title1 = ['Coil #' num2str(m) ' Image'];title(title1);
end

pause	% Press any key to continue

clc
%	Undersampling:
%
%	Choose a reduction factor = 2, we down-sample the k-space
%   data by 2 (in the y direction), which means we take out every 
%   other k-space lines. This undersampled raw data will result
%   in a reduced FOV and alias artifact in the images. In figure 2 
%   we see that every pixel in the aliased image is a linear 
%   combination of 2 pixels from the original image. The 2nd row 
%   contains the aliased images from each coil.
%

pause	% Press any key to begin

rawfull =  fftshift(ifft2(object));
coilrawfull = zeros(FOV,FOV,coiln);
for m = 1:coiln
    coilrawfull(:,:,m) = fftshift(ifft2(coilimage(:,:,m)));
end

reduc = 2;

coilimagealias = reducek(coilrawfull, reduc );
coilimagepad = zeros(FOV,FOV,coiln);

figure(2);
subplot(2,4,2:3);dispimage(abs(reduc2));title('Aliased Image (Reduction factor = 2)');

for m = 1:coiln
    coilimagepad(FOV/2-FOV/reduc/2+1:FOV/2+FOV/reduc/2,:,m) = coilimagealias(FOV/2-FOV/reduc/2+1:FOV/2+FOV/reduc/2,:,m);
    subplot(2,4,4+m);dispimage(abs(coilimagepad(:,:,m)));title1 = ['Coil #' num2str(m)];title(title1);
end

pause	% Press any key to continue

clc
%	SENSE reconstruction
%   
%   Use values from coil sensitivity map as the coefficients of
%   the linear system Ma = b : b is a 4x1 vector contains the values of
%   the same pixel of 4 aliased images. M is a 4x2 matrix; its one row 
%   contains coil sensitivity values of the 2 corresponding aliased 
%   positions from one coil. Solve this overdetermined system and we 
%   get a un-aliased image.
%

pause	% Press any key to begin

recon = senserecon(coilimagealias, coilprofile, reduc);

figure(3);
dispimage(abs(recon));title('Reconstructed Image');

pause	% Press any key to continue

clc

%	Undersampling:
%
%	This time we use a reduction factor = 4, so we down-sample the 
%   k-space data by 4 in the y direction). In the image domain the 
%   new FOV will be 1/4 the original FOV, and each pixel is a linear 
%   combination of 4 pixels from the original image. The 2nd row 
%   contains the aliased images from each coil.
%

pause	% Press any key to begin

reduc = 4;

coilimagealias = reducek(coilrawfull, reduc );
coilimagepad = zeros(FOV,FOV,coiln);

figure(4);
subplot(2,4,2:3);dispimage(abs(reduc4));title('Aliased Image (Reduction factor = 4)');

for m = 1:coiln
    coilimagepad(FOV/2-FOV/reduc/2+1:FOV/2+FOV/reduc/2,:,m) = coilimagealias(FOV/2-FOV/reduc/2+1:FOV/2+FOV/reduc/2,:,m);
    subplot(2,4,4+m);dispimage(abs(coilimagepad(:,:,m)));title1 = ['Coil #' num2str(m)];title(title1);
end

pause	% Press any key to continue

clc
%	SENSE reconstruction
%   
%   Use values from coil sensitivity map as the coefficients of
%   the linear system Ma = b : b is a 4x1 vector contains the values of
%   the same pixel of 4 aliased images. M is a 4x4 matrix; its one row 
%   contains coil sensitivity values of the 4 corresponding aliased 
%   positions from one coil. Solve this linear system and we get a 
%   un-aliased image.
%

pause	% Press any key to begin

recon = senserecon(coilimagealias, coilprofile, reduc);

figure(5);
dispimage(abs(recon));title('Reconstructed Image');


%
% 	This concludes the demo.  
%

